@@ -14,7 +14,9 @@
 
 #include <KConfig>
 #include <KConfigGroup>
+#include <KDesktopFile>
 #include <KGlobalShortcutInfo>
+#include <KIO/DesktopExecParser>
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KOpenWithDialog>
@@ -36,6 +38,7 @@ KCMKeys::KCMKeys(QObject *parent, const KPluginMetaData &metaData, const QVarian
 {
     constexpr char uri[] = "org.kde.private.kcms.keys";
     qmlRegisterUncreatableType<BaseModel>(uri, 2, 0, "BaseModel", "Can't create BaseModel");
+    qmlRegisterUncreatableMetaObject(ComponentNS::staticMetaObject, uri, 2, 0, "ComponentType", "Can't create Component namespace");
     qmlRegisterAnonymousType<ShortcutsModel>(uri, 2);
     qmlRegisterAnonymousType<FilteredShortcutsModel>(uri, 2);
     qmlProtectModule(uri, 2);
@@ -159,7 +162,7 @@ QVariantList KCMKeys::defaultSchemes() const
 
 void KCMKeys::addApplication(QQuickItem *ctx)
 {
-    auto dialog = new KOpenWithDialog;
+    KOpenWithDialog *dialog = new KOpenWithDialog();
     if (ctx && ctx->window()) {
         dialog->winId(); // so it creates windowHandle
         dialog->windowHandle()->setTransientParent(QQuickRenderControl::renderWindowFor(ctx->window()));
@@ -182,6 +185,67 @@ void KCMKeys::addApplication(QQuickItem *ctx)
     });
 }
 
+void KCMKeys::addCommand(const QString &exec)
+{
+    QString serviceName = KIO::DesktopExecParser::executableName(exec);
+    if (serviceName.isEmpty()) {
+        return;
+    }
+    QString menuId;
+    QString newPath = KService::newServicePath(false /* ignored argument */, serviceName, &menuId);
+
+    KDesktopFile desktopFile(newPath);
+    KConfigGroup cg = desktopFile.desktopGroup();
+    cg.writeEntry("Type", "Application");
+
+    QString finalExec = exec;
+
+    // If it was added as a URL, convert it to a local path, because desktop
+    // files can't take URLs in their exec keys
+    const QUrl execAsURL = QUrl(exec);
+    if (!execAsURL.scheme().isEmpty()) {
+        finalExec = execAsURL.toLocalFile();
+    }
+
+    // For the user visible name, use the executable name with any
+    // arguments appended, but with desktop-file specific expansion
+    // arguments removed. This is done to more clearly communicate the
+    // actual command used to the user and makes it easier to
+    // distinguish things like "qdbus".
+    QString name = KIO::DesktopExecParser::executableName(finalExec);
+    auto view = QStringView{finalExec}.trimmed();
+    int index = view.indexOf(QLatin1Char(' '));
+    if (index > 0) {
+        name.append(view.mid(index));
+    }
+    cg.writeEntry("Name", finalExec);
+    cg.writeEntry("Exec", finalExec);
+    cg.writeEntry("NoDisplay", true);
+    cg.writeEntry("StartupNotify", false);
+    cg.writeEntry("X-KDE-GlobalAccel-CommandShortcut", true);
+    cg.sync();
+
+    m_globalAccelModel->addApplication(newPath, name);
+}
+
+QString KCMKeys::editCommand(const QString &componentName, const QString &newExec)
+{
+    QString finalExec = newExec;
+
+    // If it was added as a URL, convert it to a local path, because desktop
+    // files can't take URLs in their exec keys
+    const QUrl execAsURL = QUrl(newExec);
+    if (!execAsURL.scheme().isEmpty()) {
+        finalExec = execAsURL.toLocalFile();
+    }
+    KDesktopFile desktopFile(componentName);
+    KConfigGroup cg = desktopFile.desktopGroup();
+    cg.writeEntry("Name", finalExec);
+    cg.writeEntry("Exec", finalExec);
+    cg.sync();
+    return finalExec;
+}
+
 QString KCMKeys::keySequenceToString(const QKeySequence &keySequence) const
 {
     return keySequence.toString(QKeySequence::NativeText);
@@ -221,7 +285,7 @@ void KCMKeys::requestKeySequence(QQuickItem *context, const QModelIndex &index,
     }
 
     qCDebug(KCMKEYS) << "Found conflict for" << newSequence << conflict;
-    const bool isStandardAction = conflict.parent().data(BaseModel::SectionRole).toString() == i18n("Common Actions");
+    const bool isStandardAction = conflict.parent().data(BaseModel::SectionRole) == ComponentType::CommonAction;
     const QString actionName = conflict.data().toString();
     const QString componentName = conflict.parent().data().toString();
     const QString keysString = newSequence.toString(QKeySequence::NativeText);
